/*****************************************************************************
** File:
**      arch_Win32Properties.h
**
** Author:
**      Daniel Vik
**
** Description:
**      blueMSX properties dialog
**
******************************************************************************
*/
#ifndef ARCH_WIN32_PROPERTIES_H
#define ARCH_WIN32_PROPERTIES_H


#include <windows.h>
#include "audioMixer.h"

typedef enum { PROP_EMULATION = 0, PROP_VIDEO, PROP_SOUND, PROP_CONTROLS } PropPage;


typedef enum { P_EMU_MSX = 0, P_EMU_MSX2, P_EMU_MSX2P, P_EMU_MSXBR, P_EMU_MSXJ, P_EMU_MSX2J } PropEmuFamily;
typedef enum { P_EMU_SPEEDX03 = 0, P_EMU_SPEEDX05, P_EMU_SPEEDX1, P_EMU_SPEEDX2, P_EMU_SPEEDX3 } PropEmuSpeed;
typedef enum { P_EMU_RAM16 = 0, P_EMU_RAM32, P_EMU_RAM64, P_EMU_RAM128, P_EMU_RAM256, P_EMU_RAM512, P_EMU_RAM1024, P_EMU_RAM2048, P_EMU_RAM4096 } PropEmuRAM;
typedef enum { P_EMU_VRAM16 = 0, P_EMU_VRAM32, P_EMU_VRAM64, P_EMU_VRAM128 } PropEmuVRAM;
typedef enum { P_EMU_SYNC1MS = 0, P_EMU_SYNCAUTO } PropEmuSync;
typedef enum { P_VIDEO_COLOR = 0, P_VIDEO_BW, P_VIDEO_GREEN } PropVideoMon;
typedef enum { P_VIDEO_PALNONE = 0, P_VIDEO_PALYC, P_VIDEO_PALNYC, P_VIDEO_PALCOMP, P_VIDEO_PALNCOMP,P_VIDEO_PALSCALE2X } PropVideoPalEmu;
typedef enum { P_VIDEO_PAL, P_VIDEO_NTSC } PropVideoType;
typedef enum { P_VIDEO_SIZEX1 = 0, P_VIDEO_SIZEX2, P_VIDEO_SIZEFULLSCREEN } PropVideoSize;
typedef enum { P_VIDEO_DRVDIRECTX = 0, P_VIDEO_DRVGDI } PropVideoDriver;
typedef enum { P_VIDEO_FSKIP0 = 0, P_VIDEO_FSKIP1, P_VIDEO_FSKIP2, P_VIDEO_FSKIP3, P_VIDEO_FSKIP4, P_VIDEO_FSKIP5 } PropVideoFrameSkip;
typedef enum { P_VIDEO_FRES320X240_16 = 0, P_VIDEO_FRES320X240_32, P_VIDEO_FRES640X480_16, P_VIDEO_FRES640X480_32 } PropVideoFullRes;
typedef enum { P_SOUND_DRVNONE = 0, P_SOUND_DRVWMM, P_SOUND_DRVDIRECTX } PropSoundDriver;
typedef enum { P_SOUND_FREQ11 = 0, P_SOUND_FREQ22, P_SOUND_FREQ44 } PropSoundFrequency;
typedef enum { P_SOUND_BUF100 = 0, P_SOUND_BUF150, P_SOUND_BUF200, P_SOUND_BUF250, P_SOUND_BUF300, P_SOUND_BUF350 } PropSoundBufSize;
typedef enum { P_SOUND_SYNCFAST = 0, P_SOUND_SYNCQADJUST, P_SOUND_SYNCEADJUST } PropSoundSync;
typedef enum { P_JOY_NONE = 0, P_JOY_NUMPAD, P_JOY_KEYSETA, P_JOY_KEYSETB, P_JOY_HW1, P_JOY_HW2, P_JOY_MOUSE } PropControlsJoy;
typedef enum { P_JOY_AFOFF = 0, P_JOY_AFSLOW, P_JOY_AFMEDIUM, P_JOY_AFFAST } PropControlsAutofire;
typedef enum { P_CHAR_EUROPEAN = 0, P_CHAR_RUSSIAN, P_CHAR_JAPANESE, P_CHAR_KOREAN } PropControlsLanguage;

#define MAX_HISTORY 10

typedef struct {
    PropEmuFamily       family;
    PropEmuSpeed        speed;
    PropEmuRAM          RAMsize;
    PropEmuVRAM         VRAMsize;
    PropEmuSync         syncMethod;
} EmulationProperties;

typedef struct {
    PropVideoMon        monType;
    PropVideoPalEmu     palEmu;
    PropVideoType       videoType;
    PropVideoSize       size;
    PropVideoDriver     driver;
    PropVideoFrameSkip  frameSkip;
    PropVideoFullRes    fullRes;
} VideoProperties;

typedef struct {
    int enable;
    int volume;
    int pan;
} MixerChannel;

enum MixerChannelType {
    MCT_PSG = 0,
    MCT_SCC,
    MCT_MSXAUDIO,
    MCT_MSXMUSIC,
    MCT_KEYBOARD,
    MCT_CASETTE,
    MCT_NUMENTRIES
};

typedef struct {
    PropSoundDriver     driver;
    PropSoundFrequency  frequency;
    PropSoundBufSize    bufSize;
    PropSoundSync       syncMethod;
    int                 stereo;
    int                 masterVolume;
    MixerChannel        mixerChannel[6]; // See MixerChannelType
    int                 logFileCount;
    char                log[MAX_PATH];
} SoundProperties;

typedef struct {
    int id;
    PropControlsJoy      type;
    PropControlsAutofire autofire;
    int keyUp;
    int keyDown;
    int keyLeft;
    int keyRight;
    int button1;
    int button2;
}  JoystickProperties;

typedef struct {
    PropControlsLanguage keySet;
} KeyboardProperties;

typedef struct {
    char slotA[MAX_PATH];
    char slotB[MAX_PATH];
    //char slotAZip[MAX_PATH];
    //char slotBZip[MAX_PATH];
} CartridgeProperties;

typedef struct {
    char slotA[MAX_PATH];
    char slotB[MAX_PATH];
    //char slotAZip[MAX_PATH];
    //char slotBZip[MAX_PATH];
    int autostartA;
} DiskdriveProperties;

typedef struct {
    char cartridgeA[MAX_HISTORY][MAX_PATH];
    char cartridgeB[MAX_HISTORY][MAX_PATH];
    char diskdriveA[MAX_HISTORY][MAX_PATH];
    char diskdriveB[MAX_HISTORY][MAX_PATH];
} FileHistory;

typedef struct {
    EmulationProperties emulation;
    VideoProperties     video;
    SoundProperties     sound;
    JoystickProperties  joy1;
    JoystickProperties  joy2;
    KeyboardProperties  keyboard;
    CartridgeProperties cartridge;
    DiskdriveProperties diskdrive;
    FileHistory         filehistory;
} Properties;

int showProperties(Properties* pProperties, HWND hwndOwner, PropPage startPage, Mixer* mixer);

Properties* propCreate();
void propSave(Properties* pProperties);
void propDestroy(Properties* pProperties);
BOOL propSetVideoSize(Properties* pProperties, PropVideoSize size);

#endif
